package driver;

import java.io.FileInputStream;
import java.io.IOException;

/**
 * utilities for dealing with binary files
 * @author Adam Gashlin
 *
 */
public final class Binary {
	
	/**
	 * prevent instantiation (this is a utility class)
	 *
	 */
	private Binary() {}
	
	/**
	 * read a big endian long integer (8 bytes) from a byte array
	 * @param buf
	 * @return
	 */
	public static final long readLong(byte [] buf) {
		return 
		   ((((long)buf[0]) << 56)&0xff00000000000000L) |
	       ((((long)buf[1]) << 48)&0x00ff000000000000L) |
	       ((((long)buf[2]) << 40)&0x0000ff0000000000L) |
	       ((((long)buf[3]) << 32)&0x000000ff00000000L) |
		   ((((long)buf[4]) << 24)&0x00000000ff000000L) |
	       ((((long)buf[5]) << 16)&0x0000000000ff0000L) |
	       ((((long)buf[6]) << 8 )&0x000000000000ff00L) |
	       ((long)buf[7])&0x00000000000000ffL;
	}
	
	/**
	 * read a big endian integer (4 bytes) from a byte array
	 * @param buf
	 * @return
	 */
	public static final int readInt(byte [] buf) {
		return ((((int)buf[0]) << 24)&0xff000000) |
		       ((((int)buf[1]) << 16)&0x00ff0000) |
		       ((((int)buf[2]) << 8 )&0x0000ff00) |
		       ((int)buf[3])&0x000000ff;
	}
	
	/**
	 * read a big endian short integer (2 bytes) from a byte array
	 * @param buf
	 * @return
	 */
	public static final short readShort(byte [] buf) {
		return (short)(((((short)buf[0]) << 8 )&0xff00) |
		      (((short)buf[1])&0x00ff));
	}
	
	/**
	 * read a big endian long integer (8 bytes) from a FileInputStream
	 * @param file
	 * @return
	 * @throws IOException
	 */
	public static final long readLong(FileInputStream file) throws IOException {
		byte [] buf = new byte[8];
		file.read(buf);
		return readLong(buf);
	}
	
	/**
	 * read a big endian integer (4 bytes) from a FileInputStream
	 * @param file
	 * @return
	 * @throws IOException
	 */
	public static final int readInt(FileInputStream file) throws IOException {
		byte [] buf = new byte[4];
		file.read(buf);
		return readInt(buf);
	}
	
	/**
	 * read a big endian short integer (2 bytes) from a FileInputStream
	 * @param file
	 * @return
	 * @throws IOException
	 */
	public static final short readShort(FileInputStream file) throws IOException {
		byte [] buf = new byte[2];
		file.read(buf);
		return readShort(buf);
	}
	
	/**
	 * read a big endian long integer (8 bytes) from the given offset of a byte array
	 * @param buf
	 * @param offset
	 * @return
	 */
	public static final long readLong(byte [] buf, int offset) {
		byte [] longbuf = new byte[8];
		System.arraycopy(buf,offset,longbuf,0,8);
		return readLong(longbuf);
	}
	
	/**
	 * read a big endian integer (4 bytes) from the given offset of a byte array
	 * @param buf
	 * @param offset
	 * @return
	 */
	public static final int readInt(byte [] buf, int offset) {
		byte [] intbuf = new byte[4];
		System.arraycopy(buf,offset,intbuf,0,4);
		return readInt(intbuf);
	}
	
	/**
	 * read a big endian short integer (2 bytes) from the given offset of a byte array
	 * @param buf
	 * @param offset
	 * @return
	 */
	public static final short readShort(byte [] buf, int offset) {
		byte [] shortbuf = new byte[2];
		System.arraycopy(buf,offset,shortbuf,0,2);
		return readShort(shortbuf);
	}
	
	/**
	 * convert an integer (4 bytes) into a series of hexadecimal digits<p>
	 * implemented so there is no need to rely on java's printf
	 * @param i
	 * @return
	 */
	public static final String intToHex(int i) {
		final String [] digits = {"0","1","2","3","4","5","6","7","8","9","A","B","C","D","E","F"};

		String s = digits[(i >> 28)&0xf]+digits[(i >> 24)&0xf]+digits[(i >> 20)&0xf]+
		digits[(i >> 16)&0xf]+digits[(i >> 12)&0xf]+digits[(i >> 8)&0xf]+digits[(i >> 4)&0xf]+
		digits[i&0xf];
		
		return s;		
	}
	
	/**
	 * zero extend a byte to an int (for reading unsigned bytes)
	 * @param b
	 * @return
	 */
	public static final int zeroExtend(byte b) {
		return ((int)b)&0xff;
	}
	
	/**
	 * zero extend a short to an int (for reading unsigned shorts)
	 * @param s
	 * @return
	 */
	public static final int zeroExtend(short s) {
		return ((int)s)&0xffff;
	}
	
	/**
	 * zero extend an int to a long (for reading unsigned ints)
	 * @param s
	 * @return
	 */
	public static final long zeroExtendtoLong(int s) {
		return ((long)s)&0x00000000ffffffffL;
	}
}
